from datetime import datetime, timezone

from lazy_tailscale.client.resources.device import Device


def device_is_online(device: Device) -> bool:
    if not device.last_seen:
        return False
    return (device.last_seen - datetime.now(timezone.utc)).total_seconds() < 500



import subprocess

from lazy_tailscale.local_client import TailscaleLocalClient, TailscaleStatus

from .exceptions import ExitNodeActivationException


class TailscaleLocalService:
    def __init__(self, client: TailscaleLocalClient):
        self.client = client

    def get_status_info(self) -> TailscaleStatus:
        return self.client.status()

    def activate_exit_node(self, hostname: str) -> None:
        cmd = ["tailscale", "set", "--exit-node", hostname]
        process = subprocess.run(cmd, timeout=3)
        if process.returncode != 0:
            raise ExitNodeActivationException()

    def deactivate_exit_node(self) -> None:
        cmd = ["tailscale", "set", "--exit-node", ""]
        process = subprocess.run(cmd, timeout=3, stderr=None, stdout=None)
        if process.returncode != 0:
            raise ExitNodeActivationException()



import platform
import re
import subprocess
from logging import getLogger

import requests

from .status import TailscaleStatus

logger = getLogger(__name__)


class TailscaleLocalClient:
    def __init__(self) -> None:
        self.session = requests.Session()
        self.base_url = None
        self.online = False
        try:
            self._connect()
        except ConnectionFailedError:
            pass

    def _connect(self) -> None:
        port, password = self._get_local_api_info()
        self.base_url = f"http://127.0.0.1:{port}/localapi/v0"
        self.session.auth = ("", password)
        self.online = True

    def _make_request(self, endpoint: str):
        if not self.online:
            self._reconnect()

        url = f"{self.base_url}/{endpoint}"
        try:
            response = self.session.get(url)
            response.raise_for_status()
            return response.json()
        except (requests.RequestException, ConnectionError):
            self.online = False
            self._reconnect()
            response = self.session.get(url)
            response.raise_for_status()
            return response.json()

    def _reconnect(self) -> None:
        try:
            self._connect()
        except ConnectionFailedError:
            raise NotConnectedError("Tailscale Local API is not reachable")

    def _get_local_api_info(self) -> tuple[str, str]:
        platform_name = platform.system().lower()
        if platform_name == "darwin":
            return self._get_macos_localapi_info()
        raise NotImplementedError(f"Local API info retrieval not implemented for {platform_name}")

    def _get_macos_localapi_info(self) -> tuple[str, str]:
        cmd = "lsof -n -a -c IPNExtension -F"
        result = subprocess.run(cmd.split(), capture_output=True, text=True)
        match = re.search(r"sameuserproof-(\d+)-([\w]+)", result.stdout)
        if not match:
            raise ConnectionFailedError("Could not find Tailscale Local API info on macOS")
        return match.group(1), match.group(2)

    def status(self) -> TailscaleStatus:
        logger.debug("Fetching Tailscale status from local API")
        try:
            return TailscaleStatus(**self._make_request("status"))
        except Exception as e:
            logger.error(f"Error fetching Tailscale status: {e}")
            raise NotConnectedError()


class ConnectionFailedError(Exception):
    pass


class NotConnectedError(Exception):
    pass



from textual.app import ComposeResult
from textual.widgets import Label


class OSLogo(Label):
    OS_LOGOS = {
        "linux": "",
        "windows": "\uf17a",
        "macos": "\uf179",
        "darwin": "\uf179",
        "ios": "\uf179",
        "android": "\uf17b",
        "freebsd": "\uf3a4",
        "openbsd": "\uf305",
    }
    DEFAULT_LOGO = "\uf128"  # Questionmark icon

    def __init__(self, os_name: str):
        super().__init__()
        self.os_name = os_name.lower()

    def compose(self) -> ComposeResult:
        logo = self.OS_LOGOS.get(self.os_name, self.DEFAULT_LOGO)
        yield Label(logo, classes="os-logo")



from textual.app import ComposeResult
from textual.widgets import Pretty, Static

from lazy_tailscale.local_client.status import PeerInfo


class ExitNodeDetail(Static):
    def __init__(self, exit_node: PeerInfo):
        super().__init__()

        self.border_title = exit_node.host_name
        self.exit_node = exit_node

    def compose(self) -> ComposeResult:
        yield Pretty(self.exit_node)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict, Field


class Latency(BaseModel):
    preferred: bool = Field(default=False, description="'true' for the node's preferred DERP server for incoming traffic.")
    latency_ms: float = Field(alias="latencyMs", description="Current latency to DERP server.")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class ClientSupports(BaseModel):
    """Identifies features supported by the client."""

    hair_pinning: Optional[bool] = Field(
        default=None, alias="hairPinning", description="True if router can route LAN connections back to LAN using global IP:port"
    )
    ipv6: bool = Field(description="True if device OS supports IPv6")
    pcp: bool = Field(description="True if PCP port-mapping service exists on router")
    pmp: bool = Field(description="True if NAT-PMP port-mapping service exists on router")
    udp: bool = Field(description="True if UDP traffic is enabled on current network")
    upnp: bool = Field(description="True if UPnP port-mapping service exists on router")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class ClientConnectivity(BaseModel):
    """Report on the device's current physical network conditions."""

    endpoints: list[str] = Field(description="Client's magicsock UDP IP:port endpoints (IPv4 or IPv6)")
    mapping_varies_by_dest_ip: bool = Field(
        default=False, alias="mappingVariesByDestIP", description="True if host's NAT mappings vary based on destination IP"
    )
    latency: dict[str, Latency] = Field(description="Map of DERP server locations and their current latency in seconds")
    client_supports: ClientSupports = Field(alias="clientSupports", description="Features supported by the client")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class PostureIdentity(BaseModel):
    """Extra identifiers from the device for posture identification."""

    disabled: Optional[bool] = None


class Device(BaseModel):
    """A Tailscale device (node/machine) in a tailnet."""

    addresses: list[str] = Field(description="List of Tailscale IP addresses (IPv4 and IPv6)")
    id: str = Field(description="Legacy identifier for the device (deprecated, use nodeId)")
    node_id: str = Field(alias="nodeId", description="Preferred identifier for the device")
    user: str = Field(description="User who registered the node")
    name: str = Field(description="MagicDNS name of the device")
    hostname: str = Field(description="Machine name in the admin console")
    client_version: str = Field(default="", alias="clientVersion", description="Version of Tailscale client software")
    update_available: Optional[bool] = Field(
        default=None, alias="updateAvailable", description="True if a client version upgrade is available"
    )
    os: str = Field(description="Operating system the device is running")
    created: Optional[datetime] = Field(default=None, description="Date the device was added to the tailnet")
    connected_to_control: bool = Field(
        alias="connectedToControl", description="Whether device recently maintained connection to control server"
    )
    last_seen: Optional[datetime] = Field(default=None, alias="lastSeen", description="When device was last connected to control server")
    key_expiry_disabled: bool = Field(alias="keyExpiryDisabled", description="True if device keys will not expire")
    expires: Optional[datetime] = Field(default=None, description="Expiration date of device's auth key")
    authorized: bool = Field(description="True if device has been authorized to join the tailnet")
    is_external: bool = Field(alias="isExternal", description="True if device is shared into tailnet but not a member")
    multiple_connections: Optional[bool] = Field(
        default=None, alias="multipleConnections", description="True if multiple devices use the same node key"
    )
    machine_key: str = Field(default="", alias="machineKey", description="For internal use, empty for external devices")
    node_key: str = Field(alias="nodeKey", description="Required for select operations like adding to locked tailnet")
    blocks_incoming_connections: bool = Field(
        alias="blocksIncomingConnections", description="True if device cannot accept connections over Tailscale"
    )
    enabled_routes: list[str] = Field(default_factory=list, alias="enabledRoutes", description="Subnet routes approved by tailnet admin")
    advertised_routes: list[str] = Field(
        default_factory=list, alias="advertisedRoutes", description="Subnets this device requests to expose"
    )
    client_connectivity: Optional[ClientConnectivity] = Field(
        default=None, alias="clientConnectivity", description="Report on device's current physical network conditions"
    )
    tags: list[str] = Field(default_factory=list, description="Tags assigned to the device for ACL purposes")
    tailnet_lock_error: Optional[str] = Field(
        default=None, alias="tailnetLockError", description="Issue with tailnet lock node-key signature"
    )
    tailnet_lock_key: str = Field(alias="tailnetLockKey", description="Node's tailnet lock key")
    ssh_enabled: Optional[bool] = Field(default=None, alias="sshEnabled", description="True if Tailscale SSH is enabled")
    posture_identity: Optional[PostureIdentity] = Field(
        default=None, alias="postureIdentity", description="Extra identifiers for device posture identification"
    )
    is_ephemeral: Optional[bool] = Field(default=None, alias="isEphemeral", description="True if the device is ephemeral")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)



from textual.app import ComposeResult
from textual.reactive import reactive
from textual.widgets import Label, Static

from lazy_tailscale.dependencies import AppDependencies
from lazy_tailscale.local_client import NotConnectedError

REFREHSH_INTERVAL_SECONDS = 10


class StatusBadge(Static):
    status_text = reactive("")
    status_class = reactive("")

    def __init__(self, deps: AppDependencies):
        super().__init__()
        self.deps = deps
        self.border_title = "Tailnet"

    def compose(self) -> ComposeResult:
        yield Label(id="tailnet-label")
        yield Label(id="status-label")

    def on_mount(self) -> None:
        self.set_interval(REFREHSH_INTERVAL_SECONDS, self.update_status)
        self.update_status()

    def update_status(self) -> None:
        try:
            status = self.deps.local_service.get_status_info()
            is_online = status.self_info.online
            tailnet = status.current_tailnet.magic_dns_suffix

            tailnet_label = self.query_one("#tailnet-label", Label)
            status_label = self.query_one("#status-label", Label)

            tailnet_label.update(f"{tailnet} | ")

            if is_online:
                status_label.update("Online")
                status_label.set_class(True, "online")
                status_label.set_class(False, "offline")
            else:
                status_label.update("Offline")
                status_label.set_class(False, "online")
                status_label.set_class(True, "offline")

        except NotConnectedError:
            tailnet_label = self.query_one("#tailnet-label", Label)
            status_label = self.query_one("#status-label", Label)

            tailnet_label.update("Not Connected | ")
            status_label.update("Offline")
            status_label.set_class(False, "online")
            status_label.set_class(True, "offline")



class ResourceNotFoundError(Exception):
    """Exception raised when a requested resource is not found."""

    msg: str


class UnauthorizedError(Exception):
    """Exception raised when authentication fails or is missing."""

    msg: str


class ServerError(Exception):
    """Exception raised when the server returns an error response."""

    msg: str


class ServerTimeoutError(Exception):
    """Exception raised when the server times out."""

    msg: str



from textual import work
from textual.app import ComposeResult
from textual.containers import VerticalScroll
from textual.message import Message
from textual.reactive import reactive
from textual.widgets import Label, LoadingIndicator, Static

from lazy_tailscale.client.resources.device import Device
from lazy_tailscale.core import utils
from lazy_tailscale.dependencies import AppDependencies
from lazy_tailscale.ui.online_indicator import OnlineIndicator
from lazy_tailscale.ui.os_logo import OSLogo
from lazy_tailscale.ui.utils import clear_screen

from ..core.exceptions import PingTimeOutException, SSHConnectionException
from .ssh_indicator import SSHIndicator


class DeviceListItem(Static, can_focus=True):
    def __init__(self, device: Device):
        super().__init__()
        self.device = device
        self.change_count = 0

    def compose(self) -> ComposeResult:
        name = self.device.name.split(".")[0]
        yield OSLogo(self.device.os)
        yield Label(name, classes="device-name")
        yield LoadingIndicator()
        yield OnlineIndicator(utils.device_is_online(self.device))
        yield SSHIndicator(self.device.ssh_enabled)

    class Focused(Message):
        def __init__(self, device: Device) -> None:
            super().__init__()
            self.device = device

    def on_focus(self) -> None:
        self.post_message(self.Focused(self.device))

    def increment_change_count(self) -> None:
        self.change_count += 1
        self.query_one(LoadingIndicator).styles.visibility = "visible"

    def decrement_change_count(self) -> None:
        self.change_count -= 1
        if self.change_count <= 0:
            self.query_one(LoadingIndicator).styles.visibility = "hidden"


class DevicesList(VerticalScroll, can_focus=False):
    BINDINGS = [
        ("p", "ping_device()", "Ping device"),
        ("s", "ssh_device()", "SSH into device"),
    ]

    devices = reactive(list)

    def __init__(self, deps: AppDependencies):
        super().__init__()
        self.deps = deps

    def on_mount(self) -> None:
        self.border_title = "[1] Devices"
        self.set_timer(0.1, self.load_devices)

    def load_devices(self):
        self.devices = self.deps.remote_service.list_devices()

    def watch_devices(self, devices: list) -> None:
        self.remove_children()

        if devices:
            for device in devices:
                self.mount(DeviceListItem(device))

            self.query_one(DeviceListItem).focus()
        else:
            self.mount(Label("Loading devices..."))

    # TODO: Create different message for errors
    class PingResult(Message):
        def __init__(self, device: Device, success: bool, response_time: float | None, address: str | None) -> None:
            super().__init__()
            self.device = device
            self.success = success
            self.response_time = response_time
            self.address = address

    class SshConnectionError(Message):
        def __init__(self, device: Device) -> None:
            super().__init__()
            self.device = device

    @work(exclusive=True, thread=True)
    async def action_ping_device(self) -> None:
        device_list_item = self.query_one("DeviceListItem:focus", DeviceListItem)
        device_list_item.increment_change_count()
        device = device_list_item.device
        try:
            address, response_time = self.deps.remote_service.ping_device(device)
            self.post_message(self.PingResult(device, True, response_time, address))
        except ValueError:
            self.post_message(self.PingResult(device, False, None, None))
        except PingTimeOutException:
            self.post_message(self.PingResult(device, False, None, None))
        finally:
            device_list_item.decrement_change_count()

    def action_ssh_device(self) -> None:
        device = self.query_one("DeviceListItem:focus", DeviceListItem).device
        with self.app.suspend():
            try:
                clear_screen()
                print(f"Trying to SSH into: {device.hostname} ...")
                self.deps.remote_service.ssh_into_device(device)
            except SSHConnectionException:
                self.post_message(self.SshConnectionError(device))



from textual import on
from textual.app import App, ComposeResult
from textual.containers import Container
from textual.css.query import NoMatches
from textual.widgets import Footer

from lazy_tailscale.dependencies import AppDependencies
from lazy_tailscale.ui.device_detail import DeviceDetail
from lazy_tailscale.ui.device_list import DeviceListItem, DevicesList
from lazy_tailscale.ui.exit_node_detai import ExitNodeDetail
from lazy_tailscale.ui.exit_node_list import ExitNodeListItem, ExitNodesList
from lazy_tailscale.ui.notification import Notification
from lazy_tailscale.ui.status_badge import StatusBadge


class TailscaleTui(App):
    CSS_PATH = "style.tcss"

    BINDINGS = [
        ("q", "quit", "Quit"),
        ("j", "cursor_down", "Down"),
        ("k", "cursor_up", "Up"),
        ("1", "focus_devices", "Focus Devices"),
        ("2", "focus_exit_nodes", "Focus Exit Nodes"),
    ]

    def __init__(self, deps: AppDependencies) -> None:
        super().__init__()
        self.dependencies = deps

    def on_mount(self) -> None:
        self.theme = "catppuccin-mocha"

    def compose(self) -> ComposeResult:
        self.notification_container = Container(id="notification-container")
        with Container(id="side-panel"):
            yield StatusBadge(self.dependencies)
            yield DevicesList(self.dependencies)
            yield ExitNodesList(self.dependencies)
        yield Container(id="detail-container")
        yield self.notification_container
        yield Footer()

    def action_cursor_down(self) -> None:
        self.screen.focus_next()

    def action_cursor_up(self) -> None:
        self.screen.focus_previous()

    def action_focus_devices(self) -> None:
        self.screen.query(DeviceListItem).first().focus()

    def action_focus_exit_nodes(self) -> None:
        try:
            self.screen.query(ExitNodeListItem).first().focus()
        except NoMatches:
            notification = Notification("No exit nodes available", variant="info")
            self.notification_container.mount(notification)

    @on(DeviceListItem.Focused)
    def focus_next_device(self, message: DeviceListItem.Focused) -> None:
        self.query_one("#detail-container").remove_children()
        self.query_one("#detail-container").mount(DeviceDetail(message.device))

    @on(DevicesList.SshConnectionError)
    def send_connection_error_notification(self, message: DevicesList.SshConnectionError) -> None:
        notification = Notification(f"   SSH into {message.device.name} failed.", variant="error")
        self.notification_container.mount(notification)

    @on(DevicesList.PingResult)
    def send_ping_result_notification(self, message: DevicesList.PingResult) -> None:
        if message.success:
            notification = Notification(f"   Ping received from {message.device.name} in {message.response_time} ms.")
        else:
            notification = Notification(f"   Ping to {message.device.name} failed.", variant="error")
        self.notification_container.mount(notification)

    @on(ExitNodeListItem.Focused)
    def focus_next_exit_node(self, message: ExitNodeListItem.Focused) -> None:
        try:
            detailed = self.query_one("#detail-container", Container)
            detailed.remove_children()
            detailed.mount(ExitNodeDetail(exit_node=message.exit_node))
        except NoMatches:
            # FIXME: This message should not be send when there are not exit nodes.
            pass



import click


from .dependencies import initialise_dependencies, get_config_path, write_api_token_to_config
from .ui.app import TailscaleTui


@click.command()
@click.argument("token", default=None, required=False)
def run(token) -> None:
    if token:
        config_path = get_config_path()
        write_api_token_to_config(config_path, token)

    dependencies = initialise_dependencies()
    app = TailscaleTui(deps=dependencies)
    app.run()



from logging import getLogger

from textual.app import ComposeResult
from textual.containers import Container, VerticalScroll
from textual.message import Message
from textual.reactive import reactive
from textual.widgets import Label, Static

from lazy_tailscale.dependencies import AppDependencies
from lazy_tailscale.local_client import NotConnectedError

from ..local_client.status import PeerInfo

logger = getLogger(__name__)
REFREHSH_INTERVAL_SECONDS = 10


class ExitNodesList(VerticalScroll, can_focus=False):
    BINDINGS = [
        ("a", "activate_exit_node", "Activate Exit Node"),
        ("d", "deactivate_exit_node", "Deactivate Exit Node"),
    ]

    peers: reactive[dict[str, PeerInfo] | None] = reactive(None, init=False)

    def __init__(self, deps: AppDependencies):
        super().__init__()
        self.deps = deps
        self.border_title = "[2] Exit Nodes"

    def on_mount(self) -> None:
        self.load_peers()
        self.set_interval(REFREHSH_INTERVAL_SECONDS, self.load_peers)

    def compose(self) -> ComposeResult:
        yield Container(id="exit-nodes-container")

    def load_peers(self) -> None:
        try:
            status = self.deps.local_service.get_status_info()
            self.peers = status.peer
        except NotConnectedError:
            logger.info("Cannot load peers: Not connected to Tailscale")
            self.peers = {}

    def watch_peers(self, peers: dict[str, PeerInfo] | None) -> None:
        container = self.query_one("#exit-nodes-container", Container)

        if not peers:
            container.remove_children()
            return

        exit_nodes = {peer.host_name: peer for peer in peers.values() if peer.exit_node_option}
        existing_items = container.query(ExitNodeListItem)

        if not existing_items:
            container.mount_all([ExitNodeListItem(peer) for peer in exit_nodes.values()])
            return

        for item in existing_items:
            if peer := exit_nodes.pop(item.peer.host_name, None):
                item.peer = peer
            else:
                item.remove()

        for new_peer in exit_nodes.values():
            container.mount(ExitNodeListItem(new_peer))

    async def action_activate_exit_node(self) -> None:
        node = self.query_one("ExitNodeListItem:focus", ExitNodeListItem).peer
        self.deps.local_service.activate_exit_node(node.host_name)
        self.load_peers()

    async def action_deactivate_exit_node(self) -> None:
        self.deps.local_service.deactivate_exit_node()
        self.load_peers()


class ExitNodeListItem(Static, can_focus=True):
    peer: reactive[PeerInfo] = reactive(None)

    def __init__(self, peer: PeerInfo):
        super().__init__()
        self.peer = peer

    def compose(self) -> ComposeResult:
        yield Label(self.peer.host_name, id="hostname")
        if self.peer.exit_node:
            yield Label(" (Active)", classes="online", id="status")

    async def watch_peer(self, _new_peer: PeerInfo) -> None:
        if self.has_focus:
            self.post_message(self.Focused(self.peer))

        await self.recompose()

    class Focused(Message):
        def __init__(self, exit_node: PeerInfo) -> None:
            super().__init__()
            self.exit_node = exit_node

    def on_focus(self) -> None:
        self.post_message(self.Focused(self.peer))



from textual.app import ComposeResult
from textual.widgets import Label


class SSHIndicator(Label):
    def __init__(self, ssh_enabled: bool | None):
        super().__init__()
        self.ssh_enabled = ssh_enabled

    def compose(self) -> ComposeResult:
        if self.ssh_enabled:
            yield Label("SSH ", classes="ssh-indicator")
        else:
            yield Label("    ", classes="ssh-indicator")



from textual.app import ComposeResult
from textual.widgets import Pretty, Static

from lazy_tailscale.client.resources.device import Device


class DeviceDetail(Static):
    def __init__(self, device: Device):
        super().__init__()

        self.border_title = device.name
        self.device = device

    def compose(self) -> ComposeResult:
        yield Pretty(self.device)



from typing import Literal

from textual.widgets import ProgressBar, Static


class Notification(Static):
    def __init__(self, message: str, duration: int = 3, variant: Literal["success", "info", "warning", "error"] = "success"):
        super().__init__()

        if variant == "success":
            self.add_class("success-notification")
        elif variant == "error":
            self.add_class("error-notification")

        self.message = message
        self.duration = duration

    def compose(self):
        progress_bar = ProgressBar(total=self.progress_duration(), show_percentage=False, show_eta=False)
        progress_bar.update(progress=self.progress_duration())
        yield Static(self.message)
        yield progress_bar

    def on_mount(self) -> None:
        self.set_timer(self.progress_duration(), lambda: self.add_class("fade-out"))
        self.set_timer(self.duration, lambda: self.remove())
        self.set_interval(0.01, self.make_progress)

    def make_progress(self) -> None:
        self.query_one(ProgressBar).advance(-0.01)

    def progress_duration(self) -> float:
        return self.duration - 0.5



import logging
import os
import sys
from dataclasses import dataclass
from logging import getLogger
from pathlib import Path

from lazy_tailscale.client import BlockingTailscaleClient
from lazy_tailscale.core.local_service import TailscaleLocalService
from lazy_tailscale.core.protocols import TailscaleLocalServiceProtocol, TailscaleRemoteServiceProtocol
from lazy_tailscale.core.remote_service import TailscaleRemoteService
from lazy_tailscale.local_client import TailscaleLocalClient

logger = getLogger(__name__)

DEFAULT_CONFIG_PATH: Path = Path.home() / ".lazy_tailscale"
API_CREDENTIALS_FILE_NAME: str = "credentials"


WELCOME_MESSAGE = """
Welcome to Lazy Tailscale!
To authenticate against the Tailscale API.
You can create an API key from the Tailscale Admin Console: `https://login.tailscale.com/admin/settings/keys`

After creating the API key set the environment variable `LAZY_TAILSCALE_API_KEY` to the value of the key.
Alternatively store the key in ~/.lazy_tailscale/credeitials.
Use `lazytailscale --token <api_key>` to store the key.
Going forward the token will be read from the config file.
"""


@dataclass
class AppDependencies:
    config_path: Path
    remote_service: TailscaleRemoteServiceProtocol
    local_service: TailscaleLocalServiceProtocol


def read_api_token_from_config(config_path: Path) -> str | None:
    try:
        with open(config_path / API_CREDENTIALS_FILE_NAME, "r") as f:
            return f.read().strip()
    except FileNotFoundError:
        logger.info(f"API credentials file not found in {config_path}.")
        return None


def write_api_token_to_config(config_path: Path, api_key: str) -> None:
    credentials_file = config_path / API_CREDENTIALS_FILE_NAME
    credentials_file.touch(mode=0o600, exist_ok=True)
    with open(credentials_file, "w") as f:
        f.write(api_key)


def get_config_path() -> Path:
    return Path(os.getenv("LAZY_TAILSCALE_CONFIG_PATH", DEFAULT_CONFIG_PATH))


def initialise_dependencies() -> AppDependencies:
    config_path = get_config_path()

    if not config_path.exists():
        logger.info(f"Creating config directory at {config_path}")
        config_path.mkdir(parents=True, exist_ok=True, mode=0o700)

    # TODO: Configure more sophisticated logging
    logging.basicConfig(
        level="DEBUG",
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        handlers=[logging.FileHandler(config_path / "app.log")],
    )

    api_key = os.getenv("LAZY_TAILSCALE_API_KEY")
    if api_key:
        logger.info("Writing API key from environment variable to config file")
        write_api_token_to_config(config_path, api_key)
    else:
        logger.info("Reading API key from config file")
        api_key = read_api_token_from_config(config_path)

    if not api_key:
        print(WELCOME_MESSAGE)
        sys.exit(1)

    client = BlockingTailscaleClient(api_key=api_key)
    local_client = TailscaleLocalClient()

    remote_service = TailscaleRemoteService(client)
    local_service = TailscaleLocalService(local_client)

    return AppDependencies(config_path=config_path, remote_service=remote_service, local_service=local_service)



from typing import Protocol

from lazy_tailscale.client.resources.device import Device
from lazy_tailscale.local_client.status import TailscaleStatus


class TailscaleRemoteServiceProtocol(Protocol):
    def list_devices(self) -> list[Device]: ...

    def ping_device(self, device: Device) -> tuple[str, int]: ...

    def ssh_into_device(self, device: Device) -> None: ...


class TailscaleLocalServiceProtocol(Protocol):
    def get_status_info(self) -> TailscaleStatus: ...

    def activate_exit_node(self, hostname: str) -> None: ...

    def deactivate_exit_node(self) -> None: ...



from requests import HTTPError, Session

from .exceptions import ServerError, UnauthorizedError
from .resources import Device


class BlockingTailscaleClient:
    BASE_URL = "https://api.tailscale.com/api/v2"

    def __init__(self, api_key: str, tailnet: str = "-"):
        self._api_key = api_key
        self._tailnet = tailnet

        self._open_session = None

    @property
    def _token_header(self):
        return {"Authorization": f"Bearer {self._api_key}"}

    @property
    def _session(self) -> Session:
        if not self._open_session:
            session = Session()
            session.headers.update(self._token_header)
            self._open_session = session

        return self._open_session

    def close_session(self) -> None:
        if self._open_session:
            self._session.close()
            self._open_session = None

    def __enter__(self):
        return self

    def __exit__(self, _exc_type, _exc_val, _exc_tb):
        if self._open_session:
            self.close_session()

    def _make_request(self, url: str):
        response = self._session.get(url)

        try:
            response.raise_for_status()
            return response.json()
        except HTTPError as e:
            if e.response.status_code == 401:
                raise UnauthorizedError("Not authorized")
            if e.response.status_code == 500:
                raise ServerError("Internal error in Tailscale server")
            raise e

    def list_devices(self, tailnet: str | None = None) -> list[Device]:
        # TODO: Add query parameters option
        tailnet = tailnet or self._tailnet
        url = f"{self.BASE_URL}/tailnet/{tailnet}/devices?fields=all"
        resp = self._make_request(url)
        json = resp["devices"]
        return [Device(**device) for device in json]

    def get_device(self, id: str) -> Device:
        url = f"{self.BASE_URL}/device/{id}"
        resp = self._session.get(url)
        return Device(**resp.json())



import re
import subprocess

from lazy_tailscale.client.blocking_client import BlockingTailscaleClient
from lazy_tailscale.client.resources.device import Device

from .exceptions import PingTimeOutException, SSHConnectionException


class TailscaleRemoteService:
    def __init__(self, client: BlockingTailscaleClient):
        self.client = client

    def list_devices(self) -> list[Device]:
        with self.client:
            devices = self.client.list_devices()
        return devices

    def _parse_ping_response(self, output: str) -> tuple[str, int]:
        pattern = r"pong from .+? \(([0-9.]+)\) .+? in (\d+)ms"

        match = re.search(pattern, output)
        if match:
            ip_address = match.group(1)
            milliseconds = int(match.group(2))
            return (ip_address, milliseconds)

        raise ValueError("Ping response format is invalid")

    def ping_device(self, device: Device) -> tuple[str, int]:
        cmd = ["tailscale", "ping", device.name]
        try:
            res = subprocess.run(cmd, capture_output=True, text=True, timeout=3)
        except subprocess.TimeoutExpired:
            raise PingTimeOutException("Ping command timed out")

        return self._parse_ping_response(res.stdout)

    def ssh_into_device(self, device: Device) -> None:
        cmd = ["ssh", "-o", "ConnectTimeout=5", device.name]
        process = subprocess.run(cmd, text=True, capture_output=False, stdout=None, stderr=None)
        if process.returncode != 0:
            raise SSHConnectionException()



from pydantic import BaseModel, ConfigDict, Field


class ClientVersion(BaseModel):
    latest_version: str = Field(alias="LatestVersion")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class CurrentTailnet(BaseModel):
    magic_dns_enabled: bool = Field(alias="MagicDNSEnabled")
    magic_dns_suffix: str = Field(alias="MagicDNSSuffix")
    name: str = Field(alias="Name")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class SelfInfo(BaseModel):
    online: bool = Field(alias="Online")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class PeerInfo(BaseModel):
    active: bool = Field(alias="Active")
    exit_node: bool = Field(alias="ExitNode")
    exit_node_option: bool = Field(alias="ExitNodeOption")
    host_name: str = Field(alias="HostName")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class TailscaleStatus(BaseModel):
    client_version: ClientVersion = Field(alias="ClientVersion")
    self_info: SelfInfo = Field(alias="Self")
    current_tailnet: CurrentTailnet = Field(alias="CurrentTailnet")
    peer: dict[str, PeerInfo] = Field(alias="Peer")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


# {'AuthURL': '',
#  'BackendState': 'Running',
#  'CertDomains': ['macbook-pro-von-pascal.taild5dd9e.ts.net'],
#  'ClientVersion': {'LatestVersion': '1.90.4'},
#  'CurrentTailnet': {'MagicDNSEnabled': True,
#                     'MagicDNSSuffix': 'taild5dd9e.ts.net',
#                     'Name': 'pascal736.github'},
#  'HaveNodeKey': True,
#  'Health': ['An update from version 1.86.4 to 1.90.4 is available.'],
#  'MagicDNSSuffix': 'taild5dd9e.ts.net',
#  'Peer': {'nodekey:0c3323a8c16c62e3131390c4f3de4b82a8de4f01bd82a9c2dad9feff7c48766c': {'Active': True,
#                                                                                        'Addrs': None,
#                                                                                        'AllowedIPs': ['100.72.118.109/32',
#                                                                                                       'fd7a:115c:a1e0::7c01:766d/128'],
#                                                                                        'Created': '2025-03-15T16:31:02.584350237Z',
#                                                                                        'CurAddr': '192.168.178.35:41641',
#                                                                                        'DNSName': 'green-meteor.taild5dd9e.ts.net.',
#                                                                                        'ExitNode': False,
#                                                                                        'ExitNodeOption': False,
#                                                                                        'HostName': 'green-meteor',
#                                                                                        'ID': 'nY8R1XGaV811CNTRL',
#                                                                                        'InEngine': True,
#                                                                                        'InMagicSock': True,
#                                                                                        'InNetworkMap': True,
#                                                                                        'LastHandshake': '2025-11-07T16:08:40.26736+01:00',
#                                                                                        'LastSeen': '2025-11-07T04:10:00.1Z',
#                                                                                        'LastWrite': '2025-11-03T07:00:13.819741166+01:00',
#                                                                                        'NoFileSharingReason': '',
#                                                                                        'OS': 'linux',
#                                                                                        'Online': True,
#                                                                                        'PeerAPIURL': ['http://100.72.118.109:50943',
#                                                                                                       'http://[fd7a:115c:a1e0::7c01:766d]:61616'],
#                                                                                        'PeerRelay': '',
#                                                                                        'PublicKey': 'nodekey:0c3323a8c16c62e3131390c4f3de4b82a8de4f01bd82a9c2dad9feff7c48766c',
#                                                                                        'Relay': 'fra',
#                                                                                        'RxBytes': 31489336,
#                                                                                        'TaildropTarget': 1,
#                                                                                        'TailscaleIPs': ['100.72.118.109',
#                                                                                                         'fd7a:115c:a1e0::7c01:766d'],
#                                                                                        'TxBytes': 29884232,
#                                                                                        'UserID': 1900669946660973,
#                                                                                        'sshHostKeys': ['ssh-rsa '
#                                                                                                        'AAAAB3NzaC1yc2EAAAADAQABAAABgQDG7nswo9XwilO5G7v/SOpoIutEyQkbKiyRDBbBbfcIDCTT8qnGpy6Dook4lm6EdkQhNg6v3SGhES0IV1lpDKSb6hG+DMqmcEJiP9I/OEqs4ej6Ug5CnjbIqy70E6DbJwhW9lYzeiowHrMCOWw1NivdGCgip5oW6BQVQyhCZN7hHeijRX0oMx5Mj7ttVNFes6YIHj7kkanUf2hHZp++KvO66DDdjty9yHu60FaVidWs3QI5fAw/2ZwIruECouOnax57cMHEuHv/+sqa9GMrE1cNgqB5sCex5T4vRCuSLF8uV84WCWF3xQZdd/0F38TV6gPB2QzD8JpRxAp5rbO5BROH+aOYI8RhCQzyWqo0Yn4bw12gBoJThMm2I6tfKNc8Wn2pWJ3U9xXpW1Nnx5s5N0RLIVG0+pCoZIvWww8Tno55ctdcw1wLqj1pK1OZqJGrk5a9RdfPU0COyUkPe0xBcA88ZAqL/pP2UyiTGTNUIDniqYBXkPQAWwdMQ4x9ON2Ges8=',
#                                                                                                        'ecdsa-sha2-nistp256 '
#                                                                                                        'AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBIKAWcfaZWA7Fg9qUk7z63hW1UfwikUS4Ha274gCfq1H4q6DbNov3J1dqUXxST9kejXUqRo/uHtJLrfLMEQ2jtg=',
#                                                                                                        'ssh-ed25519 '
#                                                                                                        'AAAAC3NzaC1lZDI1NTE5AAAAING0+qFA4cStGrwoYL1C+FJ8wxIFzCva9RdJnLz8yiF/']},
#           'nodekey:13821c023b1ea6940170207b13597cf7f8966a5aa9c713cd49b75b954fad8d25': {'Active': True,
#                                                                                        'Addrs': None,
#                                                                                        'AllowedIPs': ['100.109.141.38/32',
#                                                                                                       'fd7a:115c:a1e0::4f01:8d26/128'],
#                                                                                        'Created': '2025-03-15T16:31:01.72699161Z',
#                                                                                        'CurAddr': '192.168.178.47:41641',
#                                                                                        'DNSName': 'blue-star.taild5dd9e.ts.net.',
#                                                                                        'ExitNode': False,
#                                                                                        'ExitNodeOption': False,
#                                                                                        'HostName': 'blue-star',
#                                                                                        'ID': 'nzAvRsjrT411CNTRL',
#                                                                                        'InEngine': True,
#                                                                                        'InMagicSock': True,
#                                                                                        'InNetworkMap': True,
#                                                                                        'LastHandshake': '2025-11-07T16:07:53.82457+01:00',
#                                                                                        'LastSeen': '0001-01-01T00:00:00Z',
#                                                                                        'LastWrite': '2025-11-03T07:00:12.439020291+01:00',
#                                                                                        'NoFileSharingReason': '',
#                                                                                        'OS': 'linux',
#                                                                                        'Online': True,
#                                                                                        'PeerAPIURL': ['http://100.109.141.38:58010',
#                                                                                                       'http://[fd7a:115c:a1e0::4f01:8d26]:53470'],
#                                                                                        'PeerRelay': '',
#                                                                                        'PublicKey': 'nodekey:13821c023b1ea6940170207b13597cf7f8966a5aa9c713cd49b75b954fad8d25',
#                                                                                        'Relay': 'fra',
#                                                                                        'RxBytes': 501352,
#                                                                                        'TaildropTarget': 1,
#                                                                                        'TailscaleIPs': ['100.109.141.38',
#                                                                                                         'fd7a:115c:a1e0::4f01:8d26'],
#                                                                                        'TxBytes': 827900,
#                                                                                        'UserID': 1900669946660973,
#                                                                                        'sshHostKeys': ['ssh-rsa '
#                                                                                                        'AAAAB3NzaC1yc2EAAAADAQABAAABgQCm/4p4Vv92H8X+L17Racs8ZpwXhC36ahq1kAa8PVbrMdBCWnZoH0kSHvvtXWWTUJEcifhrAAXJYmlHtcT/RPiA+89yqRfwBCvC2qLQM/utuAkQaAEpPrZau0+9AQvnwPw02nCIC3DB5Jm1PyXJPbsRYZsUX0c/JF/g49xf6w9ZqU9XKFH1EXeaz6AQRRvyAS7l0bRQaE2sP1ZUJ2NAFeAebC3H9y43NBuN6ydgRpYAiVYPa+cZ1XKs45VB42GORYsQ67dxBDa6RYyoBMZY4XBDwJsKROBq8vAzSxiu9D0MXBaOPIyXObDV9MwwAP7RhUohCum5sdKvR24r+I0eCEwKTeRud3T3M+zVAIOvGIn+IQ22uRPFVcwYT9eHGiY1ygHPu4woncaFG/0cB56VUTsitIt21XCqsyf9Lc+XKpbGVfI0KISGK2hBoTqAiUh3xRmf99EZT9UJ88Mb3E18NPp8DQ/rMclbD5LXPv5+12TaokTJb7Ju1MB4el1ghThVhYU=',
#                                                                                                        'ecdsa-sha2-nistp256 '
#                                                                                                        'AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBGcwc8lw3pJkuC0b4hvgGi/zjje6UO1V9K64NOJSa537lUQXFCG+j6Ufmo1R2xWf4ZQXV29Y8OkxXzlO3Dg8+qU=',
#                                                                                                        'ssh-ed25519 '
#                                                                                                        'AAAAC3NzaC1lZDI1NTE5AAAAIM1AKQZBZGs4WNv0t2zV1DPSJG7QBvBThqVM3/ccSX3V']},
#           'nodekey:282ec64d4e1167607f5a97e21516c67511088f2bac47088d1ab1b4f020d99c45': {'Active': False,
#                                                                                        'Addrs': None,
#                                                                                        'AllowedIPs': ['100.89.156.70/32',
#                                                                                                       'fd7a:115c:a1e0::2001:9c47/128'],
#                                                                                        'Created': '2025-08-05T12:02:20.683252504Z',
#                                                                                        'CurAddr': '',
#                                                                                        'DNSName': 'tablet.taild5dd9e.ts.net.',
#                                                                                        'ExitNode': False,
#                                                                                        'ExitNodeOption': False,
#                                                                                        'HostName': 'localhost',
#                                                                                        'ID': 'nNkLmesPnX11CNTRL',
#                                                                                        'InEngine': False,
#                                                                                        'InMagicSock': True,
#                                                                                        'InNetworkMap': True,
#                                                                                        'KeyExpiry': '2026-02-01T12:02:20Z',
#                                                                                        'LastHandshake': '0001-01-01T00:00:00Z',
#                                                                                        'LastSeen': '2025-11-07T04:10:00.1Z',
#                                                                                        'LastWrite': '2025-11-03T05:46:50.197425416+01:00',
#                                                                                        'NoFileSharingReason': '',
#                                                                                        'OS': 'android',
#                                                                                        'Online': True,
#                                                                                        'PeerAPIURL': ['http://100.89.156.70:1',
#                                                                                                       'http://[fd7a:115c:a1e0::2001:9c47]:1'],
#                                                                                        'PeerRelay': '',
#                                                                                        'PublicKey': 'nodekey:282ec64d4e1167607f5a97e21516c67511088f2bac47088d1ab1b4f020d99c45',
#                                                                                        'Relay': 'fra',
#                                                                                        'RxBytes': 0,
#                                                                                        'TaildropTarget': 1,
#                                                                                        'TailscaleIPs': ['100.89.156.70',
#                                                                                                         'fd7a:115c:a1e0::2001:9c47'],
#                                                                                        'TxBytes': 0,
#                                                                                        'UserID': 1900669946660973},
#           'nodekey:53764601cc5aa3070998a3368084aeb04b4666a9e3352b9e967230772b7c8213': {'Active': False,
#                                                                                        'Addrs': None,
#                                                                                        'AllowedIPs': ['100.103.99.102/32',
#                                                                                                       'fd7a:115c:a1e0::9b01:6366/128'],
#                                                                                        'Created': '2024-10-23T17:54:22.697825142Z',
#                                                                                        'CurAddr': '',
#                                                                                        'DNSName': 'iphone-11.taild5dd9e.ts.net.',
#                                                                                        'ExitNode': False,
#                                                                                        'ExitNodeOption': False,
#                                                                                        'HostName': 'localhost',
#                                                                                        'ID': 'nWXpoF9Bx421CNTRL',
#                                                                                        'InEngine': False,
#                                                                                        'InMagicSock': True,
#                                                                                        'InNetworkMap': True,
#                                                                                        'KeyExpiry': '2026-01-31T14:18:46Z',
#                                                                                        'LastHandshake': '0001-01-01T00:00:00Z',
#                                                                                        'LastSeen': '2025-11-07T04:10:00.1Z',
#                                                                                        'LastWrite': '2025-11-03T06:51:45.319403583+01:00',
#                                                                                        'NoFileSharingReason': '',
#                                                                                        'OS': 'iOS',
#                                                                                        'Online': True,
#                                                                                        'PeerAPIURL': ['http://100.103.99.102:63678',
#                                                                                                       'http://[fd7a:115c:a1e0::9b01:6366]:53036'],
#                                                                                        'PeerRelay': '',
#                                                                                        'PublicKey': 'nodekey:53764601cc5aa3070998a3368084aeb04b4666a9e3352b9e967230772b7c8213',
#                                                                                        'Relay': 'fra',
#                                                                                        'RxBytes': 0,
#                                                                                        'TaildropTarget': 1,
#                                                                                        'TailscaleIPs': ['100.103.99.102',
#                                                                                                         'fd7a:115c:a1e0::9b01:6366'],
#                                                                                        'TxBytes': 0,
#                                                                                        'UserID': 1900669946660973},
#           'nodekey:a3f0949ed2566a8af6ae0017f67983db32a55c24743c33625b2350baff0fcd33': {'Active': False,
#                                                                                        'Addrs': None,
#                                                                                        'AllowedIPs': ['100.79.205.52/32',
#                                                                                                       'fd7a:115c:a1e0::ff01:cd35/128',
#                                                                                                       '0.0.0.0/0',
#                                                                                                       '::/0'],
#                                                                                        'CapMap': {'suggest-exit-node': None},
#                                                                                        'Created': '2025-03-15T16:28:59.4985702Z',
#                                                                                        'CurAddr': '',
#                                                                                        'DNSName': 'black-moon.taild5dd9e.ts.net.',
#                                                                                        'ExitNode': False,
#                                                                                        'ExitNodeOption': True,
#                                                                                        'HostName': 'black-moon',
#                                                                                        'ID': 'nFokBKmmnv11CNTRL',
#                                                                                        'InEngine': True,
#                                                                                        'InMagicSock': True,
#                                                                                        'InNetworkMap': True,
#                                                                                        'LastHandshake': '2025-11-07T15:33:19.849507+01:00',
#                                                                                        'LastSeen': '0001-01-01T00:00:00Z',
#                                                                                        'LastWrite': '2025-11-03T06:51:25.59467225+01:00',
#                                                                                        'NoFileSharingReason': '',
#                                                                                        'OS': 'linux',
#                                                                                        'Online': True,
#                                                                                        'PeerAPIURL': ['http://100.79.205.52:52825',
#                                                                                                       'http://[fd7a:115c:a1e0::ff01:cd35]:56837'],
#                                                                                        'PeerRelay': '',
#                                                                                        'PublicKey': 'nodekey:a3f0949ed2566a8af6ae0017f67983db32a55c24743c33625b2350baff0fcd33',
#                                                                                        'Relay': 'fra',
#                                                                                        'RxBytes': 892,
#                                                                                        'TaildropTarget': 1,
#                                                                                        'TailscaleIPs': ['100.79.205.52',
#                                                                                                         'fd7a:115c:a1e0::ff01:cd35'],
#                                                                                        'TxBytes': 1004,
#                                                                                        'UserID': 1900669946660973,
#                                                                                        'sshHostKeys': ['ssh-rsa '
#                                                                                                        'AAAAB3NzaC1yc2EAAAADAQABAAABgQDPJ2CjtXx44aVOE1qjGQJOyIPw8QhXydbaMT/uBj1yws8miBH6C4xVP9uPYAysDGejn6VIb//+dmovu95johuE+uKIiDy4Av9WR+YCL0gIwnTQgBmcdOIbBpCuj3fK2NhJwYGAwHi92ibZbbRZFrPbbdtIKDM2IWIJnnj2rPl7VjWV41oVjgYugTXwLbystTehQWRQut7S0EHcfEx2vNPaUvjBMpJs5LhE2hfznfK9/2S0KHBQF0NYDgN5YCRdSc/s4UtXst7V7zqNS1qXreqBvhLHqfYm4SajtL0JAhegEFZrxJDP34cZn4UfdBHfqyF0j0uoLWvw3KTlCUgzGfIdRl7zoxpAqFsPac7WlM7NgjcWEWIJmqBwOk8im43BqNgmiF/SXAs7cgCFh2J8QkrpojTvLrEIypTbtZTDNrIlUtWqQ5U1k9sg8qJ6OmNw6Q6eq66lo6L0IlWBDYoD8+eUG/dNwJ6R/cneeXgQoAVqBPKQibd5Ly16XAf2QvNaqLE=',
#                                                                                                        'ecdsa-sha2-nistp256 '
#                                                                                                        'AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBBgiZ7003iCTEx6VTu42/xmvOI1f6Z+OiTI3GalVCpSvpW6tlQu65TMDuKba0M7TkkgXbyW4FR3MAUVffFR8lEY=',
#                                                                                                        'ssh-ed25519 '
#                                                                                                        'AAAAC3NzaC1lZDI1NTE5AAAAIOlSO2FrPfTfUyad6kfXkfoI4d9de55h800oLLn3JubF']}},
#  'Self': {'Active': False,
#           'Addrs': ['87.144.255.241:59024',
#                     '[2003:cb:172e:b200:3db2:78e5:b148:33da]:41641',
#                     '10.42.1.2:41641',
#                     '192.168.178.63:41641',
#                     '[2003:cb:172e:b200:412:4b61:9b10:b23a]:41641'],
#           'AllowedIPs': ['100.119.33.10/32', 'fd7a:115c:a1e0::4301:210a/128'],
#           'CapMap': {'funnel': None,
#                      'https': None,
#                      'https://tailscale.com/cap/file-sharing': None,
#                      'https://tailscale.com/cap/funnel-ports?ports=443,8443,10000': None,
#                      'https://tailscale.com/cap/is-admin': None,
#                      'https://tailscale.com/cap/is-owner': None,
#                      'https://tailscale.com/cap/ssh': None,
#                      'https://tailscale.com/cap/tailnet-lock': None,
#                      'probe-udp-lifetime': None,
#                      'ssh-behavior-v1': None,
#                      'ssh-env-vars': None,
#                      'store-appc-routes': None},
#           'Capabilities': ['HTTPS://TAILSCALE.COM/s/DEPRECATED-NODE-CAPS#see-https://github.com/tailscale/tailscale/issues/11508',
#                            'funnel',
#                            'https',
#                            'https://tailscale.com/cap/file-sharing',
#                            'https://tailscale.com/cap/funnel-ports?ports=443,8443,10000',
#                            'https://tailscale.com/cap/is-admin',
#                            'https://tailscale.com/cap/is-owner',
#                            'https://tailscale.com/cap/ssh',
#                            'https://tailscale.com/cap/tailnet-lock',
#                            'probe-udp-lifetime',
#                            'ssh-behavior-v1',
#                            'ssh-env-vars',
#                            'store-appc-routes'],
#           'Created': '2024-10-23T17:49:40.185981501Z',
#           'CurAddr': '',
#           'DNSName': 'macbook-pro-von-pascal.taild5dd9e.ts.net.',
#           'ExitNode': False,
#           'ExitNodeOption': False,
#           'HostName': 'MacBook Pro von Pascal',
#           'ID': 'nvbJLtJirH11CNTRL',
#           'InEngine': False,
#           'InMagicSock': False,
#           'InNetworkMap': True,
#           'KeyExpiry': '2025-11-25T10:06:21Z',
#           'LastHandshake': '0001-01-01T00:00:00Z',
#           'LastSeen': '0001-01-01T00:00:00Z',
#           'LastWrite': '0001-01-01T00:00:00Z',
#           'NoFileSharingReason': '',
#           'OS': 'macOS',
#           'Online': True,
#           'PeerAPIURL': ['http://100.119.33.10:56122',
#                          'http://[fd7a:115c:a1e0::4301:210a]:53208'],
#           'PeerRelay': '',
#           'PublicKey': 'nodekey:eaadd7a870bbff5dc809c29bf609bb97e1c5da63f33f8a2aa92ba214d0093959',
#           'Relay': 'fra',
#           'RxBytes': 0,
#           'TaildropTarget': 0,
#           'TailscaleIPs': ['100.119.33.10', 'fd7a:115c:a1e0::4301:210a'],
#           'TxBytes': 0,
#           'UserID': 1900669946660973},
#  'TUN': True,
#  'TailscaleIPs': ['100.119.33.10', 'fd7a:115c:a1e0::4301:210a'],
#  'User': {'1900669946660973': {'DisplayName': 'Pascal736',
#                                'ID': 1900669946660973,
#                                'LoginName': 'Pascal736@github',
#                                'ProfilePicURL': 'https://avatars.githubusercontent.com/u/76991455?v=4'}},
#  'Version':86.4-t3149aad97-g60158502b'}



from textual.app import ComposeResult
from textual.widgets import Label


class OnlineIndicator(Label):
    def __init__(self, is_online: bool):
        super().__init__()
        self.is_online = is_online

    def compose(self) -> ComposeResult:
        if self.is_online:
            yield Label("Connected", classes="online")
        else:
            yield Label("󱐤  Offline", classes="offline")
